@@ -1,21 +1,44 @@
 /*
- * Context.java
+ * java-gnome, a UI library for writing GTK and GNOME programs from Java!
  *
- * Copyright (c) 2007-2009 Operational Dynamics Consulting Pty Ltd, and Others
- * 
- * The code in this file, and the library it is a part of, are made available
- * to you by the authors under the terms of the "GNU General Public Licence,
- * version 2" plus the "Classpath Exception" (you may link to this code as a
- * library into other programs provided you don't make a derivation of it).
- * See the LICENCE file for the terms governing usage and redistribution.
+ * Copyright © 2007-2010 Operational Dynamics Consulting, Pty Ltd and Others
+ *
+ * The code in this file, and the program it is a part of, is made available
+ * to you by its authors as open source software: you can redistribute it
+ * and/or modify it under the terms of the GNU General Public License version
+ * 2 ("GPL") as published by the Free Software Foundation.
+ *
+ * This program is distributed in the hope that it will be useful, but WITHOUT
+ * ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
+ * FITNESS FOR A PARTICULAR PURPOSE. See the GPL for more details.
+ *
+ * You should have received a copy of the GPL along with this program. If not,
+ * see http://www.gnu.org/licenses/. The authors of this program may be
+ * contacted through http://java-gnome.sourceforge.net/.
+ *
+ * Linking this library statically or dynamically with other modules is making
+ * a combined work based on this library. Thus, the terms and conditions of
+ * the GPL cover the whole combination. As a special exception (the
+ * "Claspath Exception"), the copyright holders of this library give you
+ * permission to link this library with independent modules to produce an
+ * executable, regardless of the license terms of these independent modules,
+ * and to copy and distribute the resulting executable under terms of your
+ * choice, provided that you also meet, for each linked independent module,
+ * the terms and conditions of the license of that module. An independent
+ * module is a module which is not derived from or based on this library. If
+ * you modify this library, you may extend the Classpath Exception to your
+ * version of the library, but you are not obligated to do so. If you do not
+ * wish to do so, delete this exception statement from your version.
  */
 package org.freedesktop.cairo;
 
 import org.gnome.gdk.Color;
 import org.gnome.gdk.Drawable;
+import org.gnome.gdk.EventExpose;
 import org.gnome.gdk.Pixbuf;
 import org.gnome.pango.Layout;
 import org.gnome.pango.LayoutLine;
+import org.gnome.rsvg.Handle;
 
 /**
  * Carry out drawing operations with the Cairo Graphics library. The current
@@ -31,9 +54,8 @@ import org.gnome.pango.LayoutLine;
  * {@link ImageSurface}, do your drawing, and then use Surface's writeToPNG()
  * to output your image.
  * <li>If drawing to the screen in a user interface application, construct a
- * Context using the underlying GDK Window in your Widget's
- * {@link org.gnome.gtk.Widget.ExposeEvent Widget.ExposeEvent}, and do your
- * drawing there.
+ * Context in your Widget's {@link org.gnome.gtk.Widget.ExposeEvent
+ * Widget.ExposeEvent}, and do your drawing there.
  * </ul>
  * 
  * See the links above for examples of each use case.
@@ -136,6 +158,11 @@ public class Context extends Entity
      * on. Use {@link #getTarget() getTarget()}.
      * 
      * <p>
+     * If you're drawing in an <code>Widget.ExposeEvent</code> then you're
+     * better off using the {@link Context#Context(EventExpose)
+     * Context(EventExpose)} constructor.
+     * 
+     * <p>
      * <i>Strictly speaking, this method is a part of GDK. We expose it here
      * as we are, from the Java bindings' perspective, constructing a Cairo
      * Context. So a constructor it is.</i>
@@ -152,6 +179,30 @@ public class Context extends Entity
         checkStatus();
     }
 
+    /**
+     * Construct a new "Cairo Context" during an ExposeEvent. This is the
+     * magic glue which allows you to link between GTK's Widgets and Cairo's
+     * drawing operations.
+     * 
+     * <p>
+     * This constructor takes the [org.gnome.gdk] EventExpose structure and
+     * passes is directly to some native code which constructs the Context,
+     * then <code>clip()</code>s it to the Region contained in the
+     * ExposeEvent. This isn't enough to save you running your drawing code,
+     * but it is enough to tell Cairo very early on to only actually render
+     * the part that has been damaged or exposed. This can save a lot of
+     * cycles deep down.
+     * 
+     * @since 4.0.17
+     */
+    /*
+     * Amazingly, GdkEventExpose has enough in it to construct a cairo_t.
+     */
+    public Context(EventExpose event) {
+        super(GdkCairoSupport.createContextFromExposeEvent(event));
+        checkStatus();
+    }
+
     /**
      * Makes a copy of the current state of the Context and saves it on an
      * internal stack. The saved state is recovered using {@link #restore()}.
@@ -271,7 +322,7 @@ public class Context extends Entity
     public void rotate(double r) {
         CairoContext.rotate(this, r);
     }
-    
+
     /**
      * Set the source pattern within this Context to an opaque color.
      * 
@@ -852,4 +903,77 @@ public class Context extends Entity
     public void transform(Matrix matrix) {
         CairoContext.transform(this, matrix);
     }
+
+    /**
+     * Render an SVG image to this Cairo surface.
+     * 
+     * <p>
+     * <i>In the underlying native library this is</i>
+     * <code>rsvg_handle_render_cairo()</code>. <i>We have placed the call
+     * here to align with other Cairo baesd image and text rendering
+     * methods.</i>
+     * 
+     * @since 4.0.18
+     */
+    public void showHandle(Handle graphic) {
+        CairoContext.showHandle(graphic, this);
+    }
+
+    /**
+     * Close the current path.
+     * 
+     * <p>
+     * This makes the path a closed loop, rather than it being a line with
+     * caps at each end. Call this when you're trying to close a shape.
+     * 
+     * <p>
+     * The current path begins at the point given to the last moveTo() call.
+     * If there's no current point, then this has no effect.
+     * 
+     * @since 4.0.17
+     */
+    public void closePath() {
+        CairoContext.closePath(this);
+    }
+
+    /**
+     * Create a new path within the current one. Although
+     * {@link #moveTo(double, double) moveTo()} also creates a new sub-path,
+     * this allows you to do so without needing destination co-ordinates.
+     * 
+     * @since 4.0.17
+     */
+    public void newSubPath() {
+        CairoContext.newSubPath(this);
+    }
+
+    /**
+     * Change the fill algorithm. The default is {@link FillRule#WINDING
+     * WINDING}.
+     * 
+     * @since 4.0.17
+     */
+    public void setFillRule(FillRule setting) {
+        CairoContext.setFillRule(this, setting);
+    }
+
+    /**
+     * Is the supplied point in the area that would be filled if
+     * {@link #fill() fill()} was called with the current path?
+     * 
+     * @since 4.0.17
+     */
+    public boolean inFill(double x, double y) {
+        return CairoContext.inFill(this, x, y);
+    }
+
+    /**
+     * Is the supplied point in the thickness that would be drawn if
+     * {@link #stroke() stroke()} was called with the current path?
+     * 
+     * @since 4.0.17
+     */
+    public boolean inStroke(double x, double y) {
+        return CairoContext.inStroke(this, x, y);
+    }
 }
